import java.awt.*;
import java.awt.event.*;

/**
* CPuzzelField ist ein Kontainer fuer die 9 CPuzzleElement(e), der grafisch dargestellt wird.
* Die Steuerung des Puzzels wird durch das Interface MouseListener uebernommen.
*
* @see CPuzzleElement
*
* @author Mirko Mumberg
* @version 1.0  date 07.06.1997
*/
public class CPuzzleField extends Canvas implements MouseListener
{		
	private Dimension 	m_dPrefSize;	// Die Groesse wird des Puzzelfeldes
	private CPuzzleElement 	m_Element[];	// Die 9 Puzzelteile
	private boolean m_Down, m_Up;		// fr die Mousesteuerung
	private int 		m_OldPos;	// speicher die Position der Mouse beim druecken der Maustaste.
	/**
	* Beinhaltet die jeweiligen Drehpositionen(0-3) der 9 Puzzelteile.
	*/
	public CArrayRandom 	m_PuzzleState;
	/**
	* Beinhaltet die Positionen der Puzzelteile auf dem Puzzelfeld.
	*/
	public CArrayRandom 	m_PuzzlePos;

	/**
	* Der Konstruktor initialisiert m_PuzzleState, m_PuzzlePos und die internen Variablen.
	* @param d Die Groesse des Puzzelfeldes.
	* @param _Element Ein Array mit den 9 Puzzeltteilen. 
	*/
	public CPuzzleField(Dimension d,CPuzzleElement [] _Element)
	{
		m_dPrefSize = d;
		m_Element = _Element;
		m_PuzzleState = new CArrayRandom(9);
		m_PuzzlePos = new CArrayRandom(9);
		merge();
		m_Down = false;
		m_Up = false;
		m_OldPos = -1;
		addMouseListener(this);

	}

	/**
	* Mischt das Puzzelfeld.
	*/
	public void merge()
	{
		m_PuzzleState.InitAllNumbers(3);
		m_PuzzlePos.InitDifferentNumbers();
	}

	/**
	* Um die Gre beizubehalten.
	*/
	public Dimension getPreferredSize()
	{
		return m_dPrefSize;
	}

	/**
	* Um die Gre bezubehalten.
	*/
	public Dimension getMinimumSize()
	{
		return m_dPrefSize;
	}

	/**
	* Neuzeichnen des Puzzelfeldes.
	* @param g Grafikoberflaeche, auf die gezeichnet werden soll.
	*/
	public void paint(Graphics g)
	{
		update(g);
	}

	/**
	* Neuzeichnen des Puzzelfeldes.
	* @param g Grafikoberflaeche, auf die gezeichnet werden soll.
	*/
	public void update(Graphics g)
	{
		g.drawImage(m_Element[m_PuzzlePos.Get(0)].getElement(m_PuzzleState.Get(m_PuzzlePos.Get(0))), 0, 0, this);
		g.drawImage(m_Element[m_PuzzlePos.Get(1)].getElement(m_PuzzleState.Get(m_PuzzlePos.Get(1))), 150, 0, this);
		g.drawImage(m_Element[m_PuzzlePos.Get(2)].getElement(m_PuzzleState.Get(m_PuzzlePos.Get(2))), 300, 0, this);
		g.drawImage(m_Element[m_PuzzlePos.Get(3)].getElement(m_PuzzleState.Get(m_PuzzlePos.Get(3))), 0, 150, this);
		g.drawImage(m_Element[m_PuzzlePos.Get(4)].getElement(m_PuzzleState.Get(m_PuzzlePos.Get(4))), 150, 150, this);
		g.drawImage(m_Element[m_PuzzlePos.Get(5)].getElement(m_PuzzleState.Get(m_PuzzlePos.Get(5))), 300, 150, this);
		g.drawImage(m_Element[m_PuzzlePos.Get(6)].getElement(m_PuzzleState.Get(m_PuzzlePos.Get(6))), 0, 300, this);
		g.drawImage(m_Element[m_PuzzlePos.Get(7)].getElement(m_PuzzleState.Get(m_PuzzlePos.Get(7))), 150, 300, this);
		g.drawImage(m_Element[m_PuzzlePos.Get(8)].getElement(m_PuzzleState.Get(m_PuzzlePos.Get(8))), 300, 300, this);
	}

	/**
	* Wandelt die Pixelposition(x,y) in die Puzzelteilposition (0-8) um.
	* @param x Pixel x-Position.
	* @param y Pixel y-Position.
	* @return Puzzelteilposition(0-8) oder -1 fuer ausserhalb des Puzzelfeldes.
	*/
	protected int getPos_isin(int x,int y)
	{
		if (             (x < 150) &&              (y < 150)) return 0;
		if ((149 < x) && (x < 300) &&              (y < 150)) return 1;
		if ((299 < x) && (x < 450) &&              (y < 150)) return 2;
		if (		 (x < 150) && (149 < y) && (y < 300)) return 3;
		if ((149 < x) && (x < 300) && (149 < y) && (y < 300)) return 4;
		if ((299 < x) && (x < 450) && (149 < y) && (y < 300)) return 5;
		if (		 (x < 150) && (299 < y) && (y < 450)) return 6;
		if ((149 < x) && (x < 300) && (299 < y) && (y < 450)) return 7;
		if ((299 < x) && (x < 450) && (299 < y) && (y < 450)) return 8;
		return -1;
	}

	/**
	* Dreht ein angegebenes Puzzelteil um 90 Grad.
	* @param i Position des zu drehenden Puzzelteils.
	*/
	protected void setNextPuzzleState(int i)
	{
		if (i != -1)
		{
			if (m_PuzzleState.Get(i) == 3)
				m_PuzzleState.Set(i, 0);
			else
				m_PuzzleState.Set(i, m_PuzzleState.Get(i) +1);
		}
	}

	/**
	* Vertauscht die Positionen zweier Puzzelteile.
	* @param newpos Position des zweiten Puzzelteils(m_OldPos ist das erste Puzzelteil).
	*/
	protected void ChangePos(int newpos)
	{
		if ((m_OldPos != -1) && (newpos != -1))
		{
			int tmp = m_PuzzlePos.Get(m_OldPos);
			m_PuzzlePos.Set(m_OldPos, m_PuzzlePos.Get(newpos));
			m_PuzzlePos.Set(newpos, tmp);
		}
	}

	/**
	* Vertauscht nur Puzzelteile, die neben dem Puzzelteil an m_OldPos liegen.
	* @param newpos Position des zweiten Puzzelteils(m_OldPos ist das erste Puzzelteil).
	*/
	protected void ChangePosNeighbour(int newpos)
	{
		switch(m_OldPos)
		{
			case 0 :
				if ((newpos == 1) || (newpos == 3))
					ChangePos(newpos);
				break;
			case 1 :
				if ((newpos == 0) || (newpos == 2) || (newpos == 4))
					ChangePos(newpos);
				break;
			case 2 :
				if ((newpos == 1) || (newpos == 5))
					ChangePos(newpos);
				break;
			case 3 :
				if ((newpos == 0) || (newpos == 4) || (newpos == 6))
					ChangePos(newpos);
				break;
			case 4 :
				if ((newpos == 1) || (newpos == 3) || (newpos == 5) || (newpos == 7))
					ChangePos(newpos);
				break;
			case 5 :
				if ((newpos == 2) || (newpos == 4) || (newpos == 8))
					ChangePos(newpos);
				break;
			case 6 :
				if ((newpos == 3) || (newpos == 7))
					ChangePos(newpos);
				break;
			case 7 :
				if ((newpos == 4) || (newpos == 6) || (newpos == 8))
					ChangePos(newpos);
				break;
			case 8 :
				if ((newpos == 5) || (newpos == 7))
					ChangePos(newpos);
		}
	}

	/**
	* Holt sich die Position(0-8) des Puzzelteils, auf dem die Maus steht und speichert diesen.
	*/
	public void mousePressed(MouseEvent e)
	{
		m_OldPos = getPos_isin(e.getX(), e.getY());
	}

	/**
	* Holt sich die Position(0-8) des Puzzelteils, auf dem die Maus steht. Vertauscht oder Dreht Puzzelteile in Abhaengigkeit von der alten Position (beim mousePressed gespeichert) und der jetzigen Position der Maus. Neuzeichnen des Puzzelfeldes.
	*/
	public void mouseReleased(MouseEvent e)
	{
		int NewPos = getPos_isin(e.getX(), e.getY());
		if(( m_OldPos == NewPos) && ( m_OldPos != -1))
			setNextPuzzleState(m_PuzzlePos.Get(m_OldPos));
		else
			ChangePosNeighbour(NewPos);
			m_OldPos = -1;
		repaint();
	}

	/**
	* Dieser Mausaktion wird nicht beachtet.
	*/
	public void mouseClicked(MouseEvent e) {}

	/**
	* Dieser Mausaktion wird nicht beachtet.
	*/
	public void mouseEntered(MouseEvent e) {}

	/**
	* Dieser Mausaktion wird nicht beachtet.
	*/
	public void mouseExited(MouseEvent e) {}

}
